<?php

namespace WebkokLab\Backend\Components\Tests\Proxy;

use Exception;
use WebkokLab\Lib\Entities\Range;
use WebkokLab\Lib\Entities\Test;
use WebkokLab\Lib\Models\User;

class Interpretation
{
    /**
     * @var array
     */
    private $ranges;

    /**
     * @var mixed|null
     */
    private $details;

    /**
     * @var mixed|null
     */
    private $test;

    /**
     * @var array
     */
    private $output;

    /**
     * @var User
     */
    private $user;

    /**
     * @var string
     */
    private static $header_key = '_webkok_lab_interpretations_header';

    /**
     * @var string
     */
    private static $footer_key = '_webkok_lab_interpretations_footer';

    public function __construct($test_id)
    {
        $this->setData($test_id);
        $this->generate();
    }

    /**
     * Update header
     *
     * @param $header
     */
    public static function updateHeader($header)
    {
        update_option(self::$header_key, $header);
    }

    /**
     * Update tests footer
     *
     * @param $footer
     */
    public static function updateFooter($footer)
    {
        update_option(self::$footer_key, $footer);
    }


    /**
     * Get basic header for tests
     *
     * @return false|mixed|void
     */
    public function getHeader($replace_keys = false)
    {
        $result = get_option(self::$header_key, '');
        $result = self::defaultReplaces($result);
        if (!$replace_keys) return $result;

        return self::replaceKeys($result);
    }

    /**
     * Get basic header for tests
     *
     * @return false|mixed|void
     */
    public function getDoctorDescription($replace_keys = false)
    {
        $result = $this->test->doctor_interpretation ?? '';
        $result = self::defaultReplaces($result);
        if (!$replace_keys) return $result;
        return self::replaceKeys($result);
    }

    /**
     * Get basic footer for tests
     *
     * @return false|mixed|void
     */
    public function getFooter($replace_keys = false)
    {
        $result = get_option(self::$footer_key, '');
        $result = self::defaultReplaces($result);
        if (!$replace_keys) return $result;
        return self::replaceKeys($result);
    }

    /**
     * Show test results
     */
    public function showResult()
    {
        if (empty($this->output)) {
            _e('Not found any interpretation for this results!', 'webkok-lab');
            return;
        }
        foreach ($this->output as $item) {
            if (!isset($item['lines']) || !is_array($item['lines'])) continue;
            echo "<b>" . ($item['title'] ?? '') . "</b>";
            echo "<p style='text-align: justify;'>" . ($item['description'] ?? '') . "</p>";


            $item['lines'] = array_filter($item['lines'], array($this, 'mapLines'));
            $last_group = end($item['lines']);
            $displayed_conditions = array();
            foreach ($item['lines'] as $group) {

                if (!isset($group['conditions']) || !is_array($group['conditions'])) continue;

                // conditions
                foreach ($group['conditions'] as $element_id => $condition) {

                    // this if, control show a line for one element
                    if (isset($displayed_conditions[$element_id])) continue;

                    echo "<div style='padding: 0 15px'>" . (implode(' ', $condition)) . "</div>";

                    $displayed_conditions[$element_id] = $condition;
                }

                // group gate
                /*if (isset($group['gate']) && $last_group != $group)
                    echo "<div><small>" . ($group['gate']) . "</small></div>";*/
            }

            echo '<br/>';
        }
    }

    public function mapLines($item)
    {
        try {
            return isset($item['conditions']) && is_array($item['conditions']) && !empty($item['conditions']);
        } catch (Exception $e) {
            return true;
        }
    }


    private function generate()
    {
        foreach ($this->ranges as $range)
            $this->generateRange($range);
    }

    private function generateRange($range)
    {
        try {
            if (!isset($range->groups) || !is_array($range->groups) || count($range->groups) == 0) return;
            $output = array();
            $output['title'] = $range->title;
            $output['description'] = nl2br($range->description);
            $output['lines'] = $this->generateGroups($range);

            $output = apply_filters('laboratory_outputs_ranges', $output, $range);

            $this->output[] = $output;

        } catch (Exception $e) {
            return null;
        }

    }

    private function generateConditions($range, $group)
    {
        try {
            $check_results = array();
            $result = array();
            foreach ($group->conditions as $condition_index => $condition) {
            /*    if (!isset($this->details[$condition->element_id])) throw new Exception('not found element for test!');
                if (!isset($this->details[$condition->element_id]['test_result'])) throw new Exception('not found test_result for test!');
var_dump($condition->element_id);*/
                $detail = $this->details[$condition->element_id];

                // check condition for this test
                $check_results[] = [
                    'result' => $this->checkCondition($detail, $condition),
                    'gate' => $condition->gate,
                ];

                // add result
                $gate = ($condition_index != count($group->conditions) - 1) ? $condition->gate : '';

                $element_id = $detail['element_id'];
                $test_result = $detail['test_result'];
                if (empty($test_result)) $test_result = 0;

                $line = [];
                if (!empty($detail['element_title'])){
                    $line = [
                        '<table border="0">',
                        sprintf(__('<tr><td colspan="2">%s :</td></tr>', 'webkok-lab'), $detail['element_title']),
                        sprintf(__('<tr><td width="220">&emsp;Standard Value : </td><td width="150">%s</td><td>%s</td></tr>', 'webkok-lab'), $detail['element_standard_range'], $detail['element_unit']),
                        sprintf(__('<tr><td>&emsp;Your Value : </td><td>%s</td><td>%s</td></tr>', 'webkok-lab'), $test_result, $detail['element_unit']),
                        '</table><br/>'
                    ];
                }

                $line = apply_filters('laboratory_outputs_line', $line, $detail, $condition, $this->test->id, $this->user->id);

                $result[$element_id] = $line;
            }

            $check = $this->checkConditionResults($check_results);
            if (!$check) throw new Exception('Condition failed!');
            return $result;
        } catch (Exception $e) {
            return null;
        }
    }

    private function checkCondition($detail, $condition)
    {
        try {
            if (!isset($this->details[$condition->element_id])) throw new Exception('not found element for test!');
            if (!isset($this->details[$condition->element_id]['test_result'])) throw new Exception('not found test_result for test!');

            $test_result = isset($detail['test_result']) ? $detail['test_result'] : null;
            $operator = $condition->operator;
            $amount = trim($condition->amount);

            if ($operator == '=' && $test_result == $amount) return true;
            if ($operator == '<' && $test_result < $amount) return true;
            if ($operator == '>' && $test_result > $amount) return true;
            if ($operator == '!' && $test_result != $amount) return true;
            return false;
        } catch (Exception $e) {
            return false;
        }
    }

    private function setData($test_id)
    {
        $this->ranges = Range::query()->getWithGroups();
        $this->ranges = apply_filters('laboratory_interpretation_ranges', $this->ranges);

        $this->test = Test::find($test_id);
        $this->user = new User($this->test->user_id ?? null);
        $this->details = isset($this->test) && isset($this->test->details) ? $this->test->details : null;
    }

    private function checkConditionResults(array $items)
    {
        try {
            $result = true;
            foreach ($items as $index => $item) {

                if ($index == 0) {
                    $result = $item['result'];
                    continue;
                }

                $gate = $items[$index - 1]['gate'];

                if ($gate == 'OR') {
                    $result = $result || $item['result'];
                    if ($result) return true;
                }

                if ($gate == 'AND') {
                    $result = $result && $item['result'];
                    if (!$result) return false;
                }
            }
            return $result;
        } catch (Exception $e) {
            return false;
        }
    }

    private function generateGroups($range)
    {
        try {
            $result = array();
            $groups_results = array();
            foreach ($range->groups as $group_index => $group) {
                if (!isset($group->conditions) || !is_array($group->conditions) || count($group->conditions) == 0)
                    throw new Exception('Group is empty!');

                $lines = $this->generateConditions($range, $group);
                $groups_results[] = [
                    'result' => !empty($lines),
                    'gate' => $group->gate,
                ];
                $result[] = [
                    'conditions' => $lines,
                    'gate' => $group->gate,
                ];
            }

            $check = $this->checkConditionResults($groups_results);
            if (!$check) throw new Exception('Groups failed!');

            return $result;
        } catch (Exception $e) {
            throw new Exception('Range is not true for this test!');
        }
    }

    private function replaceKeys($result)
    {
        try {
            // User info
            if (!empty($this->user)) {
                $result = str_replace('{user_name}', $this->user->name . ' ' . $this->user->family, $result);
                $result = str_replace('{user_email}', $this->user->email, $result);
                $result = str_replace('{user_phone}', $this->user->phone_number, $result);
                $result = str_replace('{user_address_default}', $this->user->address, $result);
                $result = str_replace('{user_address}', $this->user->getAddressInLines(), $result);
                $result = str_replace('{user_old}', $this->user->birth_date, $result);
                $result = str_replace('{user_pronoun}', $this->user->pronoun, $result);
                $result = str_replace('{user_gender}', $this->user->getGender(), $result);
                $result = str_replace('{user_first_name}', $this->user->name, $result);
                $result = str_replace('{user_last_name}', $this->user->family, $result);
                $result = str_replace('{user_pre_name}', $this->user->pre_name, $result);

            }

            if (!empty($this->test)) {
                $result = str_replace('{test_date}', wpkok_lab_date($this->test->date) ?? '{test_date}', $result);
                $result = str_replace('{test_date_day}', wpkok_lab_date($this->test->date, __('m.d.Y', 'webkok-lab')), $result);
                $result = str_replace('{test_id}', $this->test->id ?? '{test_id}', $result);
                $result = str_replace('{test_status}', wpkok_lab_test_status($this->test->status) ?? '{test_status}', $result);
                $result = str_replace('{test_doctor_description}', $this->test->doctor_interpretation ?? '{test_doctor_description}', $result);
            }

            $result = apply_filters('laboratory_text_replaced_keys', $result, $this->user);
            return $result;
        } catch (Exception $e) {
            return apply_filters('laboratory_text_replaced_keys', $result, $this->user);
        }
    }


    private static function defaultReplaces($result)
    {
        try {
            return str_replace('\"', '"', $result);
        } catch (Exception $e) {
            return $result;
        }
    }

    public function getOutputs()
    {
        return $this->output;
    }

}
