<?php

namespace WebkokLab\Lib;


use WebkokLab\Lib\Entities\Condition;
use WebkokLab\Lib\Entities\ConditionsGroup;
use WebkokLab\Lib\Entities\Range;
use WebkokLab\Lib\Entities\Test;
use WebkokLab\Lib\Entities\TestElement;

class Database
{
    public static function create()
    {
        try {
            self::create_tables();
            self::insert_data();
        } catch (\Exception $exception) {
            wpkok_lab_show_admin_message($exception->getMessage(), false, wpkok_lab_plugin_dir_name(__FILE__));
        }
    }

    private static function create_tables()
    {
        /** @global \wpdb $wpdb */
        global $wpdb;

        $charset_collate = $wpdb->has_cap('collation')
            ? $wpdb->get_charset_collate()
            : 'DEFAULT CHARACTER SET = utf8 COLLATE = utf8_general_ci';


        $wpdb->query(
            'CREATE TABLE IF NOT EXISTS ' . TestElement::getTableName() . ' (
                `id` int(11) NOT NULL AUTO_INCREMENT,
                `title` VARCHAR(190) NOT NULL,
                `standard_range` VARCHAR(190) NOT NULL,
                `unit` text NOT NULL,
                `color` varchar(190)  NOT NULL,
                 PRIMARY KEY (id) 
             ) ENGINE = INNODB ' . $charset_collate);

        $wpdb->query(
            'CREATE TABLE IF NOT EXISTS `' . Range::getTableName() . '` (
                `id` int(11) NOT NULL AUTO_INCREMENT,
                `description` text  NOT NULL,
                `title` varchar(250)  NOT NULL,
                PRIMARY KEY (id)
             ) ENGINE = INNODB
             ' . $charset_collate
        );

        $wpdb->query(
            'CREATE TABLE IF NOT EXISTS `' . Test::getTableName() . '` (
                `id` int(11) NOT NULL AUTO_INCREMENT,
                `user_id` bigint(20) UNSIGNED NOT NULL,
                `details` longtext  NOT NULL  ,
                `date` datetime NOT NULL,
                `status` varchar(150)  NOT NULL,
                `doctor_interpretation` text   NOT NULL,
                PRIMARY KEY (id),
                INDEX TEST_INDEX(user_id)
             ) ENGINE = INNODB
             ' . $charset_collate
        );

        $wpdb->query(
            'CREATE TABLE IF NOT EXISTS `' . ConditionsGroup::getTableName() . '` (
                `id` int(11) NOT NULL AUTO_INCREMENT,
                `range_id` int(11) NOT NULL,
                `gate` varchar(150)  NOT NULL,
                PRIMARY KEY (id) 
             ) ENGINE = INNODB
             ' . $charset_collate
        );

        $wpdb->query(
            'CREATE TABLE IF NOT EXISTS `' . Condition::getTableName() . '` (
                `id` int(11) NOT NULL AUTO_INCREMENT,
                `group_id` int(11) NOT NULL,
                `element_id` int(11) NOT NULL,
                `operator` varchar(150)  NOT NULL, 
                `amount` varchar(150)  NOT NULL, 
                `gate` varchar(150)  NOT NULL,
                PRIMARY KEY (id),
                INDEX TEST_INDEX(group_id, element_id)
             ) ENGINE = INNODB
             ' . $charset_collate
        );

        $wpdb->query(
            'ALTER TABLE ' . Condition::getTableName() . '
               ADD  CONSTRAINT `Fk_wh_ConditionGroup`  FOREIGN KEY (group_id) REFERENCES ' . ConditionsGroup::getTableName() . '(id);'
        );
        $wpdb->query(
            'ALTER TABLE ' . Test::getTableName() . ' 
              ADD  CONSTRAINT `Fk_wh_TestUser`  FOREIGN KEY (user_id) REFERENCES ' . $wpdb->prefix . 'users(ID);'
        );
        $wpdb->query(
            'ALTER TABLE ' . Condition::getTableName() . ' 
             ADD  CONSTRAINT `Fk_wh_ConditionElement`  FOREIGN KEY (element_id) REFERENCES ' . TestElement::getTableName() . '(ID);'
        );
        $wpdb->query(
            'ALTER TABLE ' . ConditionsGroup::getTableName() . ' 
             ADD  CONSTRAINT `Fk_wh_ConditionGroupRange`  FOREIGN KEY (range_id) REFERENCES ' . Range::getTableName() . '(id);'
        );
    }

    public static function insert_data()
    {
        self::insert_elements();
        self::insert_ranges();
    }

    public static function insert_ranges()
    {
        try {
            if (get_option('_webkok_lab_inserted_ranges')) return;
            global $wpdb;

            // components to elements
            $components = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wpdb->prefix}labora_component_description"));
            foreach ($components as $item) {
                $element = TestElement::find($item->component_id);
                if (empty($element)) continue;

                $range_id = Range::create([
                    'title' => $element->title  ?? '',
                    'description' => $item->description  ?? '',
                ]);
                if (empty($range_id)) continue;

                $group_id = ConditionsGroup::create([
                    'range_id' => $range_id,
                    'gate' => 'OR'
                ]);
                if (empty($group_id)) continue;

                Condition::create([
                    'group_id' => $group_id,
                    'element_id' => $item->component_id,
                    'operator' => '>',
                    'amount' => (float)$item->min,
                    'gate' => 'AND',
                ]);
                Condition::create([
                    'group_id' => $group_id,
                    'element_id' => $item->component_id,
                    'operator' => '<',
                    'amount' => (float)$item->max,
                    'gate' => 'AND',
                ]);

                // equal
                $group_id = ConditionsGroup::create([
                    'range_id' => $range_id,
                    'gate' => 'OR'
                ]);
                if (empty($group_id)) continue;

                Condition::create([
                    'group_id' => $group_id,
                    'element_id' => $item->component_id,
                    'operator' => '=',
                    'amount' => (float)$item->min,
                    'gate' => 'OR',
                ]);
                Condition::create([
                    'group_id' => $group_id,
                    'element_id' => $item->component_id,
                    'operator' => '=',
                    'amount' => (float)$item->max,
                    'gate' => 'OR',
                ]);


            }

            update_option('_webkok_lab_inserted_ranges', true);
        } catch (\Exception $e) {
            return null;
        }
    }

    public static function insert_elements()
    {
        try {
            if (get_option('_webkok_lab_inserted_data')) return;
            global $wpdb;

            // components to elements
            $components = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wpdb->prefix}labora_component"));
            foreach ($components as $item) {
                TestElement::create([
                    'id' => $item->component_id   ?? null,
                    'title' => $item->title  ?? '',
                    'standard_range' => $item->standard_range  ?? '',
                    'unit' => $item->unit  ?? '',
                    'color' => $item->color ?? '',
                ]);
            }

            update_option('_webkok_lab_inserted_data', true);
        } catch (\Exception $e) {
            return null;
        }
    }
}