<?php


namespace WebkokLab\Lib\Base;


use Exception;

abstract class Entity
{
    protected static $table;
    protected static $columns;
    protected static $schema;

    private $query;
    private $select = '*';
    private $wheres;
    private $limit;
    private $order;
    private $order_by;
    private $wpdb;
    private $leftJoins;
    private $innerJoins;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
        self::$schema = $this;
    }

    /**
     * Add select columns
     *
     * @param string $columns
     */
    public function select($columns = '*')
    {
        $this->select = $columns;
        return $this;
    }

    /**
     * Left join
     *
     * @param $table
     * @param $related_column
     * @param $column
     * @return $this
     */
    public function leftJoin($table, $related_column, $column)
    {
        $this->isColumn($column);

        $this->leftJoins[] = [
            'table' => $table,
            'related_column' => $related_column,
            'column' => $column,
        ];
        return $this;
    }

    /**
     * inner join
     *
     * @param $table
     * @param $related_column
     * @param $column
     * @return $this
     */
    public function innerJoin($table, $related_column, $column)
    {
        $this->isColumn($column);

        $this->innerJoins[] = [
            'table' => $table,
            'related_column' => $related_column,
            'column' => $column,
        ];
        return $this;
    }

    /**
     * Add where
     *
     * @param $column
     * @param $value
     * @param string $compare
     */
    public function where($column, $value, $compare = '=', $gate = 'AND')
    {
        $this->isColumn($column);

        $value = is_numeric($value) ? $value : "'" . $value . "'";

        $this->wheres[] = [
            'column' => $column,
            'value' => $value,
            'compare' => $compare,
            'gate' => $gate,
        ];
        return $this;
    }

    /**
     * Add where
     *
     * @param $column
     * @param $value
     * @param string $compare
     */
    public function orWhere($column, $value, $compare = '=')
    {
        $this->where($column, $value, $compare, 'OR');
        return $this;
    }


    /**
     * Where like
     *
     * @param $column
     * @param $keyword
     * @return Entity
     */
    public function whereLike($column, $keyword)
    {
        $this->where($column, '%'.$keyword.'%', ' LIKE ');
        return $this;
    }

    /**
     * Where like
     *
     * @param $column
     * @param $keyword
     * @return Entity
     */
    public function orWhereLike($column, $keyword)
    {
        $this->where($column, '%' . $keyword . '%', ' LIKE ', 'OR');
        return $this;
    }

    /**
     * Where not in
     *
     * @param $column
     * @param $value
     * @return $this
     * @throws \Exception
     */
    public function whereNoIn($column, $value): Entity
    {
        $this->isColumn($column);
        $column = $this->addTableNameToColumn($column);
        if (empty($value)) return $this;
        if (!is_array($value)) throw new \Exception('Where not in param is not array!');


        $value_string = sprintf("('%s')", implode("', '", $value));
        if (is_numeric($value[0])) $value_string = str_replace("'", '', $value_string);

        $this->wheres[] = [
            'column' => $column,
            'value' => $value_string,
            'compare' => ' NOT IN ',
        ];
        return $this;
    }

    /**
     * Where in
     *
     * @param $column
     * @param $value
     * @return $this
     * @throws \Exception
     */
    public function whereIn($column, $value): Entity
    {
        $this->isColumn($column);
        $column = $this->addTableNameToColumn($column);
        if (empty($value)) return $this;
        if (!is_array($value)) throw new \Exception('Where in param is not array!');


        $value_string = sprintf("('%s')", implode("', '", $value));
        if (is_numeric($value[0])) $value_string = str_replace("'", '', $value_string);

        $this->wheres[] = [
            'column' => $column,
            'value' => $value_string,
            'compare' => ' IN ',
        ];
        return $this;
    }

    /**
     * Set limit
     *
     * @param string $count
     * @return $this
     */
    public function limit($count = null)
    {
        try {
            $this->limit = $count;
            return $this;
        } catch (Exception $e) {
            return $this;
        }
    }

    /**
     * Set order
     *
     * @param string $type
     * @return $this
     */
    public function order($type = 'desc')
    {
        try {
            $type = strtolower(trim($type));
            if (!in_array($type, ['desc', 'asc'])) return $this;
            $this->order = $type;
            return $this;
        } catch (Exception $e) {
            return $this;
        }
    }

    /**
     * Set order by
     *
     * @param string $column
     * @return $this
     */
    public function orderBy($column = 'id')
    {
        try {
            $this->isColumn($column);
            $this->order_by = $column;
            return $this;
        } catch (Exception $e) {
            return $this;
        }
    }

    /**
     * Get items
     *
     * @param string $output
     * @return array|object|null
     */
    public function get($output = OBJECT)
    {
        return $this->wpdb->get_results($this->getSelectQuery(), $output);
    }

    /**
     * Get items count
     *
     * @return int
     */
    public function count()
    {
        try {
            $result = $this->get();
            return count($result);
        } catch (Exception $e) {
            return 0;
        }
    }

    /**
     * Find item
     *
     * @return array|object|null
     */
    public function first()
    {
        return $this->wpdb->get_row($this->getSelectQuery());
    }

    /**
     * Delete
     *
     * @return bool|int
     */
    public function delete()
    {
        return $this->wpdb->query(sprintf("DELETE %s FROM %s %s %s %s",
            self::getTableName(),
            self::getTableName(),
            $this->getInnerJoins(),
            $this->getLeftJoins(),
            $this->getWheres()
        ));
    }

    /**
     * Get table name
     *
     * @return string
     */
    public static function getTableName()
    {
        global $wpdb;
        return $wpdb->prefix . static::$table;
    }

    /**
     * Create
     *
     * @param $data
     * @return false|int
     */
    public static function create($data)
    {
        global $wpdb;
        if (empty(static::getTableName())) return false;
        $wpdb->insert(static::getTableName(), $data);
        return $wpdb->insert_id;
    }

    /**
     * Update
     *
     * @param $id
     * @param $data
     * @return bool|int
     */
    public static function update($id, $data)
    {
        global $wpdb;
        if (empty(static::getTableName())) return false;
        return $wpdb->update(static::getTableName(), $data, ['id' => $id]);
    }

    /**
     * @return Entity
     */
    public static function query()
    {
        $class = get_called_class();
        return new $class;
    }

    /**
     * Find item
     *
     * @param $id
     * @return array|object|null
     */
    public static function find($id)
    {
        return self::query()->where('id', $id)->first();
    }

    private function getLeftJoins()
    {
        $result = '';
        if (empty($this->leftJoins)) return '';
        $table_name = self::getTableName();
        foreach ($this->leftJoins as $index => $join) {
            $result .= " LEFT JOIN {$join['table']} ON {$join['related_column']} = $table_name.{$join['column']} ";
        }

        return $result;
    }

    private function getInnerJoins()
    {
        $result = '';
        if (empty($this->innerJoins)) return '';
        $table_name = self::getTableName();
        foreach ($this->innerJoins as $index => $join) {
            $result .= " INNER JOIN {$join['table']} ON {$join['related_column']} = $table_name.{$join['column']} ";
        }

        return $result;
    }

    private function getSelectQuery()
    {

        return sprintf("SELECT %s FROM %s %s %s %s %s %s",
            $this->getSelect(),
            self::getTableName(),
            $this->getInnerJoins(),
            $this->getLeftJoins(),
            $this->getWheres(),
            $this->getOrder(),
            $this->getLimit()
        );
    }

    private function getOrder()
    {
        $order = $this->order ?? ' DESC ';
        $order_by = $this->order_by ?? ' ' . self::getTableName() . '.id ';
        return " ORDER BY $order_by $order ";
    }

    private function getLimit()
    {
        if (empty($this->limit) || !is_numeric($this->limit)) return '';
        return ' LIMIT ' . $this->limit;
    }

    private function getWheres()
    {
        $result = '';
        if (empty($this->wheres)) return '';

        foreach ($this->wheres as $index => $where) {
            $result .= "{$where['column']}{$where['compare']}{$where['value']}";
            if ($index < count($this->wheres) - 1) {
                $gate = $where['gate'] ?? 'AND';
                $result .= ' ' . $gate . ' ';
            }
        }

        return ' WHERE ' . $result;
    }

    private function getSelect()
    {
        return $this->select;
    }

    private function isColumn($column)
    {
        try {
            if (!$column) throw new Exception('Not found column');
            if (!in_array($column, static::$columns)) throw new Exception('Not found column');
        } catch (Exception $e) {
            throw new Exception(self::getTableName() . ' :Not found column: ' . $column);
        }
    }

    private function addTableNameToColumn($column)
    {
        if (strpos($column, '.') !== false) return $column;
        return self::getTableName() . '.' . $column;
    }

}

